@@ -1,7 +1,7 @@
 /*
  * PosixSystem.cpp
  *
- * Copyright (C) 2009-16 by RStudio, Inc.
+ * Copyright (C) 2009-17 by RStudio, Inc.
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -41,6 +41,8 @@
 
 #ifdef __APPLE__
 #include <mach-o/dyld.h>
+#include <sys/proc_info.h>
+#include <libproc.h>
 #endif
 
 #ifndef __APPLE__
@@ -57,6 +59,8 @@
 #include <boost/algorithm/string/replace.hpp>
 #include <boost/algorithm/string/split.hpp>
 
+#include <core/RegexUtils.hpp>
+#include <core/Algorithm.hpp>
 #include <core/Error.hpp>
 #include <core/Log.hpp>
 #include <core/FilePath.hpp>
@@ -719,11 +723,318 @@ Error terminateProcess(PidType pid)
       return Success();
 }
 
+std::vector<SubprocInfo> getSubprocessesViaPgrep(PidType pid)
+{
+   std::vector<SubprocInfo> subprocs;
+
+   // pgrep -P ppid -l returns 0 if there are matches, non-zero
+   // otherwise; output is one line per direct child process,
+   // for example:
+   //
+   // 23432 sleep
+   // 23433 mycommand
+   shell_utils::ShellCommand cmd("pgrep");
+   cmd << "-P" << pid << "-l";
+
+   core::system::ProcessOptions options;
+   options.detachSession = true;
+
+   core::system::ProcessResult result;
+   Error error = runCommand(shell_utils::sendStdErrToStdOut(cmd),
+                            options,
+                            &result);
+   if (error)
+   {
+      LOG_ERROR(error);
+      return subprocs;
+   }
+
+   if (result.exitStatus == 0)
+   {
+      boost::regex re("^(\\d+)\\s+(.+)");
+      std::vector<std::string> lines = algorithm::split(result.stdOut, "\n");
+      BOOST_FOREACH(const std::string& line, lines)
+      {
+         if (boost::algorithm::trim_copy(line).empty())
+         {
+            continue;
+         }
+
+         boost::smatch matches;
+         if (regex_utils::match(line, matches, re))
+         {
+            SubprocInfo info;
+            info.pid = safe_convert::stringTo<PidType>(matches[1], -1);
+            if (info.pid != -1)
+            {
+               info.exe = matches[2];
+               subprocs.push_back(info);
+            }
+         }
+         else
+         {
+            std::string msg = "Unrecognized output from pgrep -l: '";
+            msg += line;
+            msg += "'";
+            LOG_ERROR_MESSAGE(msg);
+         }
+      }
+   }
+   return subprocs;
+}
+
+#ifdef __APPLE__ // Mac-specific subprocess detection
+
+std::vector<SubprocInfo> getSubprocessesMac(PidType pid)
+{
+   std::vector<SubprocInfo> subprocs;
+
+   int result = proc_listchildpids(pid, NULL, 0);
+   if (result > 0)
+   {
+      std::vector<PidType> buffer(result, 0);
+      result = proc_listchildpids(pid, &buffer[0], buffer.size() * sizeof(PidType));
+      for (int i = 0; i < result; i++)
+      {
+         PidType childPid = buffer[i];
+         if (childPid == 0)
+            continue;
+
+         SubprocInfo info;
+         info.pid = childPid;
+
+         // Try to get exe
+         std::string path(PROC_PIDPATHINFO_MAXSIZE, '\0');
+         proc_pidpath(childPid, &path[0], path.length());
+         path.resize(strlen(&path[0]));
+         if (!path.empty())
+         {
+            core::FilePath exePath(path);
+            info.exe = exePath.filename();
+         }
+         subprocs.push_back(info);
+      }
+   }
+   return subprocs;
+}
+
+#else
+
+std::vector<SubprocInfo> getSubprocessesViaProcFs(PidType pid)
+{
+   std::vector<SubprocInfo> subprocs;
+
+   core::FilePath procFsPath("/proc");
+   if (!procFsPath.exists())
+   {
+      return getSubprocessesViaPgrep(pid);
+   }
+
+   // We iterate all /proc/###/stat files, where ### is a process id.
+   //
+   // The parent pid is the fourth field (whitespace separated) in the
+   // single-line of the stat file. The first field is an int, second field
+   // is a string enclosed in parenthesis (...), the third is a single
+   // character, and the fourth is the parent pid (int). There are numerous
+   // fields after that, all ints of varying sizes.
+   //
+   // The trick is that the third field can contain arbitrary text,
+   // including whitespace and more parenthesis, inside its surrounding
+   // parenthesis. The safe way to parse this is to search the file
+   // in reverse for the closing parenthesis, then seek forward until we
+   // reach the first integer character.
+   //
+   // An example:
+   //    4075 (My )(great Program) S 4074 ....
+
+   std::vector<FilePath> children;
+   Error error = procFsPath.children(&children);
+   if (error)
+   {
+      LOG_ERROR(error);
+      return subprocs;
+   }
+
+   BOOST_FOREACH(const FilePath& child, children)
+   {
+      // only interested in the numeric directories (pid)
+      std::string filename = child.filename();
+      bool isNumber = true;
+      for (std::string::const_iterator k = filename.begin(); k != filename.end(); ++k)
+      {
+         if (!isdigit(*k))
+         {
+            isNumber = false;
+            break;
+         }
+      }
+
+      if (!isNumber)
+         continue;
+
+      // load the stat file
+      std::string contents;
+      FilePath statFile(child.complete("stat"));
+      Error error = rstudio::core::readStringFromFile(statFile, &contents);
+      if (error)
+      {
+         continue;
+      }
+
+      size_t closingParen = contents.find_last_of(')');
+      if (closingParen == std::string::npos)
+      {
+         LOG_ERROR_MESSAGE("no closing parenthesis");
+         continue;
+      }
+
+      size_t i = contents.find_first_of("0123456789", closingParen);
+      if (i == std::string::npos)
+      {
+         LOG_ERROR_MESSAGE("no integer after closing parenthesis");
+         continue;
+      }
+
+      size_t j = contents.find_first_not_of("0123456789", i);
+      if (j == std::string::npos)
+      {
+         LOG_ERROR_MESSAGE("no non-int after first int");
+         continue;
+      }
+
+      size_t ppidLen = j - i;
+      PidType ppid = safe_convert::stringTo<PidType>(contents.substr(i, ppidLen), -1);
+      if (ppid == -1)
+      {
+         LOG_ERROR_MESSAGE("unrecognized parent process id");
+         continue;
+      }
+      if (ppid != pid)
+      {
+         continue;
+      }
+
+      size_t openParen = contents.find_first_of('(');
+      if (openParen == std::string::npos)
+      {
+         LOG_ERROR_MESSAGE("no opening parenthesis");
+         continue;
+      }
+      if (openParen < 2) // at a minimum, "# (foo)"
+      {
+         LOG_ERROR_MESSAGE("no pid before exe name");
+         continue;
+      }
+      if (closingParen < openParen)
+      {
+         LOG_ERROR_MESSAGE("closing paren before open paren");
+         continue;
+      }
+
+      SubprocInfo info;
+      info.exe = contents.substr(openParen + 1, closingParen - openParen - 1);
+      info.pid = safe_convert::stringTo<PidType>(contents.substr(0, openParen - 1), -1);
+      if (info.pid == -1)
+      {
+         LOG_ERROR_MESSAGE("unrecognized child process id");
+         continue;
+      }
+      subprocs.push_back(info);
+   }
+
+   return subprocs;
+}
+#endif // !__APPLE__
+
+std::vector<SubprocInfo> getSubprocesses(PidType pid)
+{
+#ifdef __APPLE__
+   return getSubprocessesMac(pid);
+#else // Linux
+   return getSubprocessesViaProcFs(pid);
+#endif
+}
+
+FilePath currentWorkingDirViaLsof(PidType pid)
+{
+   // lsof -a -p PID -d cwd -Fn
+   //
+   shell_utils::ShellCommand cmd("lsof");
+   cmd << shell_utils::EscapeFilesOnly;
+   cmd << "-a";
+   cmd << "-p" << pid;
+   cmd << "-d cwd";
+   cmd << "-Fn";
+
+   core::system::ProcessOptions options;
+   core::system::ProcessResult result;
+   Error error = runCommand(cmd, options, &result);
+   if (error)
+   {
+      LOG_ERROR(error);
+      return FilePath();
+   }
+   if (result.exitStatus != 0)
+      return FilePath();
+
+   // lsof outputs multiple lines, which varies by platform. We want the one
+   // starting with lowercase 'n', after that is the current working directory.
+   size_t pos = 0;
+   while (pos != std::string::npos && pos < result.stdOut.length())
+   {
+      if (result.stdOut.at(pos) == 'n')
+      {
+         pos++;
+         size_t finalPos = result.stdOut.find_first_of('\n', pos);
+         if (finalPos != std::string::npos)
+            return FilePath(result.stdOut.substr(pos, finalPos - pos));
+         else
+            return FilePath(result.stdOut.substr(pos));
+      }
+
+      // next line
+      pos = result.stdOut.find_first_of('\n', pos);
+      pos++;
+   }
+
+   return FilePath();
+}
+
+#ifndef __APPLE__
+FilePath currentWorkingDirViaProcFs(PidType pid)
+{
+   core::FilePath procFsPath("/proc");
+   if (!procFsPath.exists())
+   {
+      return currentWorkingDirViaLsof(pid);
+   }
+
+   std::string procId = safe_convert::numberToString(pid);
+   if (procId.empty())
+      return FilePath();
+
+   // /proc/PID/cwd is a symbolic link to the process' current working directory
+   FilePath pidPath = procFsPath.complete(procId).complete("cwd");
+   if (pidPath.isSymlink())
+      return pidPath.resolveSymlink();
+   else
+      return FilePath();
+}
+#endif // !__APPLE__
+
+FilePath currentWorkingDir(PidType pid)
+{
+#ifdef __APPLE__
+   return currentWorkingDirViaLsof(pid);
+#else
+   return currentWorkingDirViaProcFs(pid);
+#endif
+}
 
 Error daemonize()
 {
    // fork
-   pid_t pid = ::fork();
+   PidType pid = ::fork();
    if (pid < 0)
       return systemError(errno, ERROR_LOCATION); // fork error
    else if (pid > 0)
@@ -919,7 +1230,7 @@ core::Error pidof(const std::string& process, std::vector<PidType>* pPids)
    return Success();
 }
 
-Error processInfo(const std::string& process, std::vector<ProcessInfo>* pInfo)
+Error processInfo(const std::string& process, std::vector<ProcessInfo>* pInfo, ProcessFilter filter)
 {
    // clear the existing process info
    pInfo->clear();
@@ -968,8 +1279,14 @@ Error processInfo(const std::string& process, std::vector<ProcessInfo>* pInfo)
          if (pos != std::string::npos)
             cmdline = cmdline.substr(0, pos);
 
+         // confirm the stat file exists for this pid
+         boost::format statFmt("/proc/%1%/stat");
+         FilePath statFile = FilePath(boost::str(statFmt % pDirent->d_name));
+         if (!statFile.exists())
+            continue;
+
          // check if this is the process we are filtering on
-         if (FilePath(cmdline).filename() == process)
+         if (process.empty() || FilePath(cmdline).filename() == process)
          {
             // stat the file to determine it's owner
             struct stat st;
@@ -990,11 +1307,45 @@ Error processInfo(const std::string& process, std::vector<ProcessInfo>* pInfo)
                continue;
             }
 
+            // read the stat fields for other relevant process info
+            std::string statStr;
+            error = core::readStringFromFile(statFile, &statStr);
+            if (error)
+            {
+               LOG_ERROR(error);
+               continue;
+            }
+
+            std::vector<std::string> statFields;
+            boost::algorithm::split(statFields, statStr,
+                                    boost::is_any_of(" "),
+                                    boost::algorithm::token_compress_on);
+
+            if (statFields.size() < 5)
+            {
+               LOG_WARNING_MESSAGE("Expected at least 5 stat fields but read: " + safe_convert::numberToString<size_t>(statFields.size()));
+               continue;
+            }
+
+            // get process parent id
+            std::string ppidStr = statFields[3];
+            pid_t ppid = safe_convert::stringTo<pid_t>(ppidStr, -1);
+
+            // get process group id
+            std::string pgrpStr = statFields[4];
+            pid_t pgrp = safe_convert::stringTo<pid_t>(pgrpStr, -1);
+
             // add a process info
             ProcessInfo pi;
             pi.pid = pid;
+            pi.ppid = ppid;
+            pi.pgrp = pgrp;
             pi.username = user.username;
-            pInfo->push_back(pi);
+
+            if (!filter || filter(pi))
+            {
+               pInfo->push_back(pi);
+            }
          }
       }
    }
@@ -1006,6 +1357,14 @@ Error processInfo(const std::string& process, std::vector<ProcessInfo>* pInfo)
    return Success();
 }
 
+bool isProcessRunning(pid_t pid)
+{
+   // the posix standard way of checking if a process
+   // is running is to send the 0 signal to it
+   int result = kill(pid, 0);
+   return result == 0;
+}
+
 #else
 core::Error pidof(const std::string& process, std::vector<PidType>* pPids)
 {
@@ -1028,19 +1387,55 @@ core::Error pidof(const std::string& process, std::vector<PidType>* pPids)
    return Success();
 }
 
-Error processInfo(const std::string& process, std::vector<ProcessInfo>* pInfo)
+Error processInfo(const std::string& process, std::vector<ProcessInfo>* pInfo, ProcessFilter filter)
 {
-   std::vector<PidType> pids;
-   Error error = pidof(process, &pids);
+   // use ps to capture process info
+   // output format
+   // USER:PID:PPID:PGID:PROCNAME
+   // we use a colon as the separator as it is not a valid path character in OSX
+   std::string cmd = process.empty() ? "ps acxj | awk '{OFS=\":\"; print $1,$2,$3,$4,$10}'"
+                                     : "ps acxj | awk '{OFS=\":\"; if ($10==\"" +
+                                        process + "\") print $1,$2,$3,$4,$10}'";
+
+   core::system::ProcessResult result;
+   Error error = core::system::runCommand(cmd,
+                                          core::system::ProcessOptions(),
+                                          &result);
    if (error)
       return error;
 
-   pInfo->clear();
-   BOOST_FOREACH(PidType pid, pids)
+   // parse into ProcessInfo
+   std::vector<std::string> lines;
+   boost::algorithm::split(lines,
+                           result.stdOut,
+                           boost::algorithm::is_any_of("\n"));
+
+   BOOST_FOREACH(const std::string& line, lines)
    {
-      ProcessInfo pi;
-      pi.pid = pid;
-      pInfo->push_back(pi);
+      if (line.empty()) continue;
+       
+      std::vector<std::string> lineInfo;
+      boost::algorithm::split(lineInfo,
+                              line,
+                              boost::algorithm::is_any_of(":"));
+
+      if (lineInfo.size() < 5)
+      {
+         LOG_WARNING_MESSAGE("Exepcted 5 items from ps output but received: " + safe_convert::numberToString<size_t>(lineInfo.size()));
+         continue;
+      }
+
+      ProcessInfo procInfo;
+      procInfo.username = lineInfo[0];
+      procInfo.pid = safe_convert::stringTo<pid_t>(lineInfo[1], 0);
+      procInfo.ppid = safe_convert::stringTo<pid_t>(lineInfo[2], 0);
+      procInfo.pgrp = safe_convert::stringTo<pid_t>(lineInfo[3], 0);
+
+      // check to see if this process info passes the filter criteria
+      if (!filter || filter(procInfo))
+      {
+         pInfo->push_back(procInfo);
+      }
    }
 
    return Success();
@@ -1115,7 +1510,7 @@ void printCoreDumpable(const std::string& context)
    ostr << "Core Dumpable (" << context << ")" << std::endl;
 
    // ulimit
-   RLimitType rLimitSoft, rLimitHard;
+   RLimitType rLimitSoft = 0, rLimitHard = 0;
    Error error = getResourceLimit(core::system::CoreLimit,
                                   &rLimitSoft, &rLimitHard);
    if (error)
@@ -1278,7 +1673,7 @@ Error launchChildProcess(std::string path,
                          ProcessConfigFilter configFilter,
                          PidType* pProcessId)
 {
-   pid_t pid = ::fork();
+   PidType pid = ::fork();
 
    // error
    if (pid < 0)
@@ -1444,7 +1839,130 @@ Error launchChildProcess(std::string path,
    return Success() ;
 }
 
+// simple cass to encapsulate parent-child
+// relationship of processes
+struct ProcessTreeNode
+{
+   boost::shared_ptr<ProcessInfo> data;
+   std::vector<boost::shared_ptr<ProcessTreeNode> > children;
+};
 
+// process tree, indexed by pid
+typedef std::map<pid_t, boost::shared_ptr<ProcessTreeNode> > ProcessTreeT;
+
+void createProcessTree(const std::vector<ProcessInfo>& processes,
+                       ProcessTreeT *pOutTree)
+{
+   // first pass, create the nodes in the tree
+   BOOST_FOREACH(const ProcessInfo& process, processes)
+   {
+      ProcessTreeT::iterator iter = pOutTree->find(process.pid);
+      if (iter == pOutTree->end())
+      {
+         // process not found, so create a new entry for it
+         boost::shared_ptr<ProcessTreeNode> nodePtr = boost::shared_ptr<ProcessTreeNode>(
+                                                         new ProcessTreeNode());
+
+         nodePtr->data = boost::shared_ptr<ProcessInfo>(new ProcessInfo(process));
+
+         (*pOutTree)[process.pid] = nodePtr;
+      }
+   }
+
+   // second pass, link the nodes together
+   BOOST_FOREACH(ProcessTreeT::value_type& element, *pOutTree)
+   {
+      pid_t parent = element.second->data->ppid;
+      ProcessTreeT::iterator iter = pOutTree->find(parent);
+
+      // if we cannot find the parent in the tree, move on
+      if (iter == pOutTree->end())
+         continue;
+
+      // add this node to its parent's children
+      iter->second->children.push_back(element.second);
+   }
+}
+
+void getChildren(const boost::shared_ptr<ProcessTreeNode>& node,
+                 std::vector<ProcessInfo> *pOutChildren)
+{
+   BOOST_FOREACH(const boost::shared_ptr<ProcessTreeNode>& child, node->children)
+   {
+      pOutChildren->push_back(*child->data.get());
+      getChildren(child, pOutChildren);
+   }
+}
+
+Error getChildProcesses(std::vector<ProcessInfo> *pOutProcesses)
+{
+   return getChildProcesses(::getpid(), pOutProcesses);
+}
+
+Error getChildProcesses(pid_t pid,
+                        std::vector<ProcessInfo> *pOutProcesses)
+{
+   if (!pOutProcesses)
+      return systemError(EINVAL, ERROR_LOCATION);
+
+   // get all processes
+   std::vector<ProcessInfo> processes;
+   Error error = processInfo("", &processes);
+   if (error) return error;
+
+   // build a process tree of the processes
+   ProcessTreeT tree;
+   createProcessTree(processes, &tree);
+
+   // return just the children of the specified process
+   ProcessTreeT::const_iterator iter = tree.find(pid);
+   if (iter == tree.end())
+      return Success();
+
+   const boost::shared_ptr<ProcessTreeNode>& rootNode = iter->second;
+   getChildren(rootNode, pOutProcesses);
+
+   return Success();
+}
+
+Error terminateChildProcesses()
+{
+   return terminateChildProcesses(SIGTERM);
+}
+
+Error terminateChildProcesses(int signal)
+{
+   return terminateChildProcesses(::getpid(), signal);
+}
+
+Error terminateChildProcesses(pid_t pid,
+                              int signal)
+{
+   std::vector<ProcessInfo> childProcesses;
+   Error error = getChildProcesses(pid, &childProcesses);
+   if (error)
+      return error;
+
+   BOOST_FOREACH(const ProcessInfo& process, childProcesses)
+   {
+      if (::kill(process.pid, signal) != 0)
+      {
+         // On the Mac some child processes exist as a side effect of the
+         // call to getChildProcesses (sh, ps, awk). This results in a
+         // regular stream of error messages at the dev console, so we
+         // suppress that particular error to prevent bogus error states.
+         Error error = systemError(errno, ERROR_LOCATION);
+#ifdef __APPLE__
+         if (error.code() != boost::system::errc::no_such_process)
+#endif
+            LOG_ERROR(error);
+      }
+   }
+
+   // the actual kill is best effort
+   // so return success regardless
+   return Success();
+}
 
 bool isUserNotFoundError(const Error& error)
 {
