@@ -161,6 +161,26 @@ const int kActiveConnectionsChanged = 142;
 const int kConnectionOpened = 143;
 const int kNotebookRangeExecuted = 144;
 const int kChunkExecStateChanged = 145;
+const int kNavigateShinyFrame = 146;
+const int kUpdateNewConnectionDialog = 147;
+const int kProjectTemplateRegistryUpdated = 148;
+const int kTerminalSubprocs = 149;
+const int kPackageExtensionIndexingCompleted = 150;
+const int kRStudioAPIShowDialog = 151;
+const int kRStudioAPIShowDialogCompleted = 152;
+const int kObjectExplorerEvent = 153;
+const int kSendToTerminal = 154;
+const int kClearTerminal = 155;
+const int kAddTerminal = 156;
+const int kActivateTerminal = 157;
+const int kTerminalCwd = 158;
+const int kAdminNotification = 159;
+const int kRequestDocumentSave = 160;
+const int kRequestDocumentSaveCompleted = 161;
+const int kRequestOpenProject = 162;
+const int kOpenFileDialog = 163;
+const int kRemoveTerminal = 164;
+const int kShowPageViewerEvent = 165;
 }
 
 void ClientEvent::init(int type, const json::Value& data)
@@ -344,10 +364,6 @@ std::string ClientEvent::typeName() const
          return "rmd_render_output";
       case client_events::kRmdRenderCompleted:
          return "rmd_render_completed";
-      case client_events::kRmdTemplateDiscovered:
-         return "rmd_template_discovered";
-      case client_events::kRmdTemplateDiscoveryCompleted:
-         return "rmd_template_discovery_completed";
       case client_events::kRmdShinyDocStarted:
          return "rmd_shiny_doc_started";
       case client_events::kRmdRSConnectDeploymentOutput:
@@ -442,6 +458,46 @@ std::string ClientEvent::typeName() const
          return "notebook_range_executed";
       case client_events::kChunkExecStateChanged:
          return "chunk_exec_state_changed";
+      case client_events::kNavigateShinyFrame:
+         return "navigate_shiny_frame";
+      case client_events::kUpdateNewConnectionDialog:
+         return "update_new_connection_dialog";
+      case client_events::kProjectTemplateRegistryUpdated:
+         return "project_template_registry_updated";
+      case client_events::kTerminalSubprocs:
+         return "terminal_subprocs";
+      case client_events::kPackageExtensionIndexingCompleted:
+         return "package_extension_indexing_completed";
+      case client_events::kRStudioAPIShowDialog:
+         return "rstudioapi_show_dialog";
+      case client_events::kRStudioAPIShowDialogCompleted:
+         return "rstudioapi_show_dialog_completed";
+      case client_events::kObjectExplorerEvent:
+         return "object_explorer_event";
+      case client_events::kSendToTerminal:
+         return "send_to_terminal";
+      case client_events::kClearTerminal:
+         return "clear_terminal";
+      case client_events::kAddTerminal:
+         return "add_terminal";
+      case client_events::kActivateTerminal:
+         return "activate_terminal";
+      case client_events::kTerminalCwd:
+         return "terminal_cwd";
+      case client_events::kAdminNotification:
+         return "admin_notification";
+      case client_events::kRequestDocumentSave:
+         return "request_document_save";
+      case client_events::kRequestDocumentSaveCompleted:
+         return "request_document_save_completed";
+      case client_events::kRequestOpenProject:
+         return "request_open_project";
+      case client_events::kOpenFileDialog:
+         return "open_file_dialog";
+      case client_events::kRemoveTerminal:
+         return "remove_terminal";
+      case client_events::kShowPageViewerEvent:
+         return "show_page_viewer";
       default:
          LOG_WARNING_MESSAGE("unexpected event type: " + 
                              safe_convert::numberToString(type_));
