@@ -35,7 +35,7 @@ void checkQuotaStatus();
 } // namespace quotas
 } // namespace files
 } // namepace handlers
-} // namesapce session
+} // namespace session
 } // namespace rstudio
 
 #endif // SESSION_SESSION_FILES_QUOTAS_HPP
