@@ -46,6 +46,7 @@
 #include <r/RJson.hpp>
 #include <r/RUtil.hpp>
 #include <r/ROptions.hpp>
+#include <r/RRoutines.hpp>
 
 #include <session/SessionModuleContext.hpp>
 #include <session/SessionSourceDatabase.hpp>
@@ -63,6 +64,35 @@ namespace html_preview {
 
 namespace {
 
+void enqueHTMLPreviewSucceeded(const std::string& title,
+                               const std::string& previewUrl,
+                               const FilePath& sourceFile,
+                               const FilePath& htmlFile,
+                               bool enableFileLabel,
+                               bool enableSaveAs,
+                               bool enableReexecute)
+{
+   using namespace module_context;
+   json::Object resultJson;
+   resultJson["succeeded"] = true;
+   resultJson["title"] = title;
+   resultJson["preview_url"] = previewUrl;
+   if (!sourceFile.empty())
+      resultJson["source_file"] = module_context::createAliasedPath(sourceFile);
+   else
+      resultJson["source_file"] = json::Value();
+   if (!htmlFile.empty())
+      resultJson["html_file"] = module_context::createAliasedPath(htmlFile);
+   else
+      resultJson["html_file"] = json::Value();
+   resultJson["enable_file_label"] = enableFileLabel;
+   resultJson["enable_saveas"] = enableSaveAs;
+   resultJson["enable_reexecute"] = enableReexecute;
+   resultJson["previously_published"] = !previousRpubsUploadId(htmlFile).empty();
+   ClientEvent event(client_events::kHTMLPreviewCompletedEvent, resultJson);
+   module_context::enqueClientEvent(event);
+}
+
 class HTMLPreview : public async_r::AsyncRProcess
 {
 public:
@@ -389,9 +419,11 @@ private:
       markCompleted();
       outputFile_ = outputFile;
 
-      enqueHTMLPreviewSucceeded(kHTMLPreview "/",
+      enqueHTMLPreviewSucceeded("RStudio: Preview HTML",
+                                kHTMLPreview "/",
                                 targetFile(),
                                 htmlPreviewFile(),
+                                true,
                                 isMarkdown(),
                                 !isNotebook());
    }
@@ -418,25 +450,6 @@ private:
       module_context::enqueClientEvent(event);
    }
 
-   static void enqueHTMLPreviewSucceeded(const std::string& previewUrl,
-                                         const FilePath& sourceFile,
-                                         const FilePath& htmlFile,
-                                         bool enableSaveAs,
-                                         bool enableRefresh)
-   {
-      using namespace module_context;
-      json::Object resultJson;
-      resultJson["succeeded"] = true;
-      resultJson["source_file"] = module_context::createAliasedPath(sourceFile);
-      resultJson["html_file"] = module_context::createAliasedPath(htmlFile);
-      resultJson["preview_url"] = previewUrl;
-      resultJson["enable_saveas"] = enableSaveAs;
-      resultJson["enable_refresh"] = enableRefresh;
-      resultJson["previously_published"] = !previousRpubsUploadId(htmlFile).empty();
-      ClientEvent event(client_events::kHTMLPreviewCompletedEvent, resultJson);
-      module_context::enqueClientEvent(event);
-   }
-
    static FilePath createOutputFile()
    {
       return module_context::tempFile("html_preview", "htm");
@@ -470,6 +483,8 @@ std::string deriveNotebookPath(const std::string& path)
 // serve the web content back)
 boost::shared_ptr<HTMLPreview> s_pCurrentPreview_;
 
+// current page_viewer html file
+
 bool isPreviewRunning()
 {
    return s_pCurrentPreview_ && s_pCurrentPreview_->isRunning();
@@ -710,7 +725,7 @@ Error createNotebook(const json::JsonRpcRequest& request,
 bool requiresHighlighting(const std::string& htmlOutput)
 {
    boost::regex hlRegex("<pre><code class=\"(r|cpp)\"");
-   return boost::regex_search(htmlOutput, hlRegex);
+   return regex_utils::search(htmlOutput, hlRegex);
 }
 
 
@@ -964,6 +979,114 @@ void handlePreviewRequest(const http::Request& request,
    }
 }
 
+SEXP rs_showPageViewer(SEXP urlSEXP, SEXP titleSEXP, SEXP selfContainedSEXP)
+{
+   try
+   {
+      // terminate any running preview
+      if (isPreviewRunning())
+         s_pCurrentPreview_->terminate();
+
+      // get parameters
+      std::string url = r::sexp::safeAsString(urlSEXP);
+      std::string title = r::sexp::safeAsString(titleSEXP);
+      bool selfContained = r::sexp::asLogical(selfContainedSEXP);
+
+      // paths we will forward via event
+      FilePath filePath, viewerFilePath;
+
+      // check for url
+      if (boost::algorithm::starts_with(url, "http"))
+      {
+         // verify local url only
+         boost::regex re("^http[s]?://(?:localhost|127\\.0\\.0\\.1).*$");
+         boost::smatch match;
+         if (!regex_utils::search(url, match, re))
+         {
+            throw r::exec::RErrorException(
+              "Only localhost URLs are allowed in the page viewer");
+         }
+
+         // port mapping for RStudio Server
+         url = module_context::mapUrlPorts(url);
+      }
+
+      // otherwise it's a file in the filesystem
+      else
+      {
+         // get full path to file
+         filePath = module_context::resolveAliasedPath(url);
+
+         // make it self_contained if it isn't already
+         if (!selfContained)
+         {
+            // create temp file to write standalone html to (place it within a temp dir
+            // so the default download file name is pretty)
+            FilePath viewerTempDir = module_context::tempFile("page-viewer", "dir");
+            Error error = viewerTempDir.ensureDirectory();
+            if (error)
+               throw r::exec::RErrorException(r::endUserErrorMessage(error));
+            viewerFilePath = viewerTempDir.childPath(filePath.filename());
+
+            // create base64 encoded version
+            error = module_context::createSelfContainedHtml(filePath, viewerFilePath);
+            if (error)
+               throw r::exec::RErrorException(r::endUserErrorMessage(error));
+         }
+         // if it's already self contained then just set the viewerFilePath to filePath
+         else
+         {
+             viewerFilePath = filePath;
+         }
+          
+         // set url to localhost previewer
+         std::string tempPath = viewerFilePath.relativePath(module_context::tempDir());
+         url = module_context::sessionTempDirUrl(tempPath);
+      }
+
+      // emit show page viewer event
+      json::Object data;
+      data["path"] = viewerFilePath.absolutePath();
+      data["encoding"] = "UTF-8";
+      data["is_markdown"] = false;
+      data["requires_knit"] = false;
+      data["is_notebook"] = false;
+      ClientEvent event(client_events::kShowPageViewerEvent, data);
+      module_context::enqueClientEvent(event);
+
+      // we need to give the first event time to be delivered so that
+      // the preview succeeded event handler is set up
+      using namespace boost::posix_time;
+      boost::this_thread::sleep(milliseconds(200));
+
+      // emit html preview completed event
+      enqueHTMLPreviewSucceeded(
+         title,               // title
+         url,                 // preview url
+         filePath,            // original source file
+         viewerFilePath,      // file to show as preview
+         false,               // file label
+         !filePath.empty(),   // save as
+         false                // re-execute
+       );
+
+      // return the path to the viewer file if it's a file, otherwise
+      // return the URL
+      r::sexp::Protect rProtect;
+      if (!viewerFilePath.empty())
+         return r::sexp::create(viewerFilePath.absolutePath(), &rProtect);
+      else
+         return r::sexp::create(url, &rProtect);
+   }
+   catch(r::exec::RErrorException e)
+   {
+      r::exec::error(e.message());
+   }
+
+   return R_NilValue;
+}
+
+
    
 } // anonymous namespace
    
@@ -1018,6 +1141,8 @@ core::json::Object capabilitiesAsJson()
 
 Error initialize()
 {  
+   RS_REGISTER_CALL_METHOD(rs_showPageViewer, 3);
+
    using boost::bind;
    using namespace module_context;
    ExecBlock initBlock ;
@@ -1036,6 +1161,6 @@ Error initialize()
 
 } // namespace html_preview
 } // namespace modules
-} // namesapce session
+} // namespace session
 } // namespace rstudio
 
