@@ -122,6 +122,50 @@ bool isLocalURL(const std::string& url,
    return false;
 }
 
+class EscapeHttpdQuery
+{
+public:
+   std::string operator()(const boost::smatch& m)
+   {
+      std::string query = m[2];
+      if (query.find('<') != std::string::npos)
+         query = string_utils::htmlEscape(query);
+
+      return m[1] + query + m[3];
+   }
+};
+
+std::string normalizeHttpdSearchContent(const std::string& content)
+{
+   return boost::regex_replace(
+            content,
+            boost::regex("(The search string was <b>\")(.*)(\"</b>)"),
+            EscapeHttpdQuery());
+}
+
+template <typename F>
+bool isHttpdErrorPayload(SEXP payloadSEXP, F accessor)
+{
+   for (int i = 0; i < r::sexp::length(payloadSEXP); i++)
+   {
+      std::string line = r::sexp::asString(accessor(payloadSEXP, i));
+      if (line.find("<title>R: httpd error</title>") != std::string::npos)
+         return true;
+   }
+
+   return false;
+}
+
+bool isHttpdErrorPayload(SEXP payloadSEXP)
+{
+   switch (TYPEOF(payloadSEXP))
+   {
+   case STRSXP : return isHttpdErrorPayload(payloadSEXP, STRING_ELT);
+   case VECSXP : return isHttpdErrorPayload(payloadSEXP, VECTOR_ELT);
+   default     : return false;
+   }
+}
+
 
 // hook the browseURL function to look for calls to the R internal http
 // server. for custom URLs remap the address to remote and then fire
@@ -317,7 +361,6 @@ void setDynamicContentResponse(const std::string& content,
    http::NullOutputFilter nullFilter;
    setDynamicContentResponse(content, request, nullFilter, pResponse);
 }
-   
 
 template <typename Filter>
 void handleHttpdResult(SEXP httpdSEXP, 
@@ -336,7 +379,7 @@ void handleHttpdResult(SEXP httpdSEXP,
    const char * const kTextHtml = "text/html";
    std::string contentType(kTextHtml);
    std::vector<std::string> headers;
-   
+
    // if present, second element is content type
    if (LENGTH(httpdSEXP) > 1) 
    {
@@ -374,7 +417,14 @@ void handleHttpdResult(SEXP httpdSEXP,
    // payload = string
    if ((TYPEOF(payloadSEXP) == STRSXP || TYPEOF(payloadSEXP) == VECSXP) &&
         LENGTH(payloadSEXP) > 0)
-   { 
+   {
+      // handle httpd errors (returned as specially constructed payload)
+      if (isHttpdErrorPayload(payloadSEXP))
+      {
+         pResponse->setError(http::status::NotFound, "URL '" + request.uri() + "' not found");
+         return;
+      }
+
       // get the names and the content string
       SEXP namesSEXP = r::sexp::getNames(httpdSEXP);
       std::string content;
@@ -382,6 +432,10 @@ void handleHttpdResult(SEXP httpdSEXP,
          content = r::sexp::asString(STRING_ELT(payloadSEXP, 0));
       else if (TYPEOF(payloadSEXP) == VECSXP)
          content = r::sexp::asString(VECTOR_ELT(payloadSEXP, 0));
+
+      // normalize search result output
+      if (boost::algorithm::iends_with(request.path(), "/search"))
+         content = normalizeHttpdSearchContent(content);
       
       // check for special file returns
       std::string fileName ;
