@@ -35,7 +35,7 @@ core::Error initialize();
                        
 } // namespace lists
 } // namespace modules
-} // namesapce session
+} // namespace session
 } // namespace rstudio
 
 #endif // SESSION_LISTS_HPP
