@@ -88,24 +88,33 @@ void writeDocToJson(boost::shared_ptr<SourceDocument> pDoc,
    }
    (*pDocJson)["notebook"] = notebook;
    
-   // discover project-specific settings when available
+   // discover project-specific settings when available (only applied when
+   // opening a file that belongs to a project separate from the active project)
    r_util::RProjectConfig projConfig;
-   FilePath docPath = module_context::resolveAliasedPath(pDoc->path());
-   bool hasConfig = false;
+   bool useConfig = false;
    
-   if (projects::projectContext().hasProject() &&
-       docPath.isWithin(projects::projectContext().directory()))
+   if (!pDoc->path().empty())
    {
-      projConfig = projects::projectContext().config();
-      hasConfig = true;
-   }
-   else
-   {
-      Error error = r_util::findProjectConfig(docPath, &projConfig);
-      hasConfig = !error;
+      FilePath docPath = module_context::resolveAliasedPath(pDoc->path());
+
+      if (projects::projectContext().hasProject() &&
+          docPath.isWithin(projects::projectContext().directory()))
+      {
+         // this file belongs to the active project: do nothing here and let
+         // other machinery handle setting of project-specific options
+         useConfig = false;
+      }
+      else
+      {
+         Error error = r_util::findProjectConfig(
+                  docPath,
+                  module_context::userHomePath(),
+                  &projConfig);
+         useConfig = !error;
+      }
    }
    
-   if (hasConfig)
+   if (useConfig)
    {
       json::Object projConfigJson;
       
