@@ -34,7 +34,7 @@ core::Error initialize();
 
 } // namespace mathjax
 } // namespace modules
-} // namesapce session
+} // namespace session
 } // namespace rstudio
 
 #endif /* SESSION_MODULES_MATHJAX_HPP */
