@@ -1,7 +1,7 @@
 /*
  * StringUtil.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2009-17 by RStudio, Inc.
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -83,7 +83,20 @@ public class StringUtil
       else 
          return (seconds / 3600) + " hour" + ((seconds / 3600) == 1 ? "" : "s");
    }
-   
+
+   // Return current time as a timestamp (yyyy/m/d hh:mm:ss)
+   public static native String getTimestamp() /*-{
+      var now = new Date();
+      var date = [ now.getFullYear(), now.getMonth() + 1, now.getDate() ];
+      var time = [ now.getHours(), now.getMinutes(), now.getSeconds() ];
+      for (var i = 1; i < 3; i++ ) {
+         if (time[i] < 10) {
+            time[i] = "0" + time[i];
+         }
+      }
+      return date.join("/") + " " + time.join(":");
+   }-*/;
+    
    // Given a raw size, convert it to a human-readable value 
    // (e.g. 11580 -> "11.3 KB"). Note that this routine must generally avoid
    // implicit casts and use only ints; GWT's JavaScript compiler will truncate
@@ -515,6 +528,32 @@ public class StringUtil
       
    }
    
+   /**
+    * Given a URL, attempt to infer and return the hostname from the URL. The URL is always 
+    * presumed to have a hostname (if it doesn't, the first component of the path will be treated
+    * as the host name
+    * 
+    * @param url URL to parse
+    * @return Host name and port, as a string.
+    */
+   public static String getHostFromUrl(String url)
+   {
+      // no work to do
+      if (url.indexOf('/') == -1)
+         return url;
+      
+      // presume no protocol; if present, skip those slashes
+      int slashes = 0;
+      if (url.contains("://"))
+         slashes += 2;
+      
+      // split on slashes and return first component
+      String[] parts = url.split("/");
+      if (parts.length < slashes)
+         return url;
+      return parts[slashes];
+   }
+    
    public static String ensureSurroundedWith(String string, char chr)
    {
       if (isNullOrEmpty(string))
@@ -1083,7 +1122,15 @@ public class StringUtil
    public static final native String normalizeNewLines(String string) /*-{
       return string.replace(/\r\n|\n\r|\r/g, "\n");
    }-*/;
-   
+
+   /**
+    * Convert string line endings to carriage returns to mimic keyboard entry
+    * of text on Windows.
+    */
+   public static final native String normalizeNewLinesToCR(String string) /*-{
+      return string.replace(/\r\n|\n\r|\n/g, "\r");
+   }-*/;
+    
    public static final native JsArrayString split(String string, String delimiter) /*-{
       return string.split(delimiter);
    }-*/;
@@ -1124,6 +1171,11 @@ public class StringUtil
 
       return ((crc ^ (-1)) >>> 0).toString(16);
    }-*/;
+
+   // Count newlines in a string
+   public static native int newlineCount(String str) /*-{
+      return (str.match(/\n/g)||[]).length;
+   }-*/;
    
    private static final NumberFormat FORMAT = NumberFormat.getFormat("0.#");
    private static final NumberFormat PRETTY_NUMBER_FORMAT = NumberFormat.getFormat("#,##0.#####");
