@@ -32,6 +32,7 @@ import com.google.gwt.user.client.*;
 import com.google.gwt.user.client.ui.Label;
 import com.google.gwt.user.client.ui.RootPanel;
 import com.google.gwt.user.client.ui.UIObject;
+import com.google.gwt.user.client.ui.Widget;
 
 import org.rstudio.core.client.BrowseCap;
 import org.rstudio.core.client.Debug;
@@ -74,6 +75,13 @@ public class DomUtils
          element.focus();
    }-*/;
 
+   /**
+    * Trim excess lines from the beginning of the text of an element.
+    * 
+    * @param element The element to trim lines from.
+    * @param linesToTrim The number of lines to trim.
+    * @return Number of lines trimmed
+    */
    public static int trimLines(Element element, int linesToTrim)
    {
       return trimLines(element.getChildNodes(), linesToTrim);
@@ -228,7 +236,7 @@ public class DomUtils
             return 0;
       }
    }
-
+   
    private static int countLinesInternal(Text textNode, boolean pre)
    {
       if (!pre)
@@ -269,6 +277,30 @@ public class DomUtils
    {
       impl.focus(element, alwaysDriveSelection);
    }
+   
+   public static boolean isFocusable(Element element) 
+   {
+      // If it has a non-negative tab index, it can be focused
+      if (element.getTabIndex() >= 0)
+         return true;
+      
+      // Otherwise, determine whether an item can be focused based
+      // on its tag name.
+      String tagName = element.getTagName().toLowerCase();
+      if (tagName.equals("a"))
+      {
+         return element.hasAttribute("href");
+      }
+      else if (tagName.equals("input") || tagName.equals("select") ||
+               tagName.equals("textarea") || tagName.equals("button"))
+      {
+         return !element.hasAttribute("disabled");
+      }
+      else
+      {
+         return false;
+      }
+   }
 
    public static native boolean hasFocus(Element element) /*-{
       return element === $doc.activeElement;
@@ -878,9 +910,16 @@ public class DomUtils
    }
    
    public static Element findParentElement(Element el,
+                                           ElementPredicate predicate)
+   {
+      return findParentElement(el, false, predicate);
+   }
+   
+   public static Element findParentElement(Element el,
+                                           boolean includeSelf,
    	                                     ElementPredicate predicate)
    {
-      Element parent = el.getParentElement();
+      Element parent = includeSelf ? el : el.getParentElement();
       while (parent != null)
       {
          if (predicate.test(parent))
@@ -1009,5 +1048,17 @@ public class DomUtils
             style);
    }
    
+   public static void disableAutoBehavior(Element ele)
+   {
+      ele.setAttribute("autocomplete", "off");
+      ele.setAttribute("autocorrect", "off");
+      ele.setAttribute("autocapitalize", "off");
+   }
+   
+   public static void disableAutoBehavior(Widget w)
+   {
+      disableAutoBehavior(w.getElement());
+   }
+   
    public static final int ESTIMATED_SCROLLBAR_WIDTH = 19;
 }
