@@ -34,6 +34,7 @@ import org.rstudio.core.client.events.TabClosingHandler;
 import org.rstudio.core.client.events.TabReorderEvent;
 import org.rstudio.core.client.events.TabReorderHandler;
 import org.rstudio.core.client.js.JsObject;
+import org.rstudio.core.client.resources.ImageResource2x;
 import org.rstudio.core.client.theme.res.ThemeResources;
 import org.rstudio.core.client.theme.res.ThemeStyles;
 import org.rstudio.studio.client.RStudioGinjector;
@@ -129,6 +130,8 @@ public class DocTabLayoutPanel
             DOM.sinkBitlessEvent(tabBar, "dragend");
             DOM.sinkBitlessEvent(tabBar, "dragleave");
             DOM.sinkBitlessEvent(tabBar, "drop");
+            DOM.sinkBitlessEvent(tabBar, "mousewheel");
+            DOM.sinkBitlessEvent(tabBar, "wheel");
             Event.setEventListener(tabBar, dragManager_);
          }
       });
@@ -492,6 +495,22 @@ public class DocTabLayoutPanel
                endDrag(event, ACTION_CANCEL);
             }
          }
+         else if (event.getType() == "wheel" ||
+                  event.getType() == "mousewheel")
+         {
+            // extract the delta from the wheel event (note that this could be
+            // zero)
+            JsObject evt = event.cast();
+            double delta = evt.getDouble(event.getType() == "wheel" ?
+                  "deltaY" : "wheelDeltaY");
+            
+            // translate wheel scroll into tab selection; don't wrap
+            int idx = getSelectedIndex();
+            if (delta > 0 && idx < (getWidgetCount() - 1))
+               selectTab(idx + 1);
+            else if (delta < 0 && idx > 0)
+               selectTab(idx - 1);
+         }
       }
 
       @Override
@@ -1040,8 +1059,14 @@ public class DocTabLayoutPanel
          contentPanel_.setStylePrimaryName(styles_.tabLayoutCenter());
          contentPanel_.setVerticalAlignment(HasVerticalAlignment.ALIGN_MIDDLE);
 
-         if (icon != null)
-            contentPanel_.add(imageForIcon(icon));
+         if (icon != null) {
+            Image tabImage = imageForIcon(icon);
+            tabImage.addStyleName(ThemeStyles.INSTANCE.tabIcon());
+            // Trivia: setDraggable(Element.DRAGGABLE_FALSE) sets
+            // "draggable" to "true".
+            tabImage.getElement().setAttribute("draggable", "false");
+            contentPanel_.add(tabImage);
+         }
 
          label_ = new Label(title, false);
          label_.addStyleName(styles_.docTabLabel());
@@ -1049,7 +1074,7 @@ public class DocTabLayoutPanel
 
          appendDirtyMarker();
 
-         Image img = new Image(ThemeResources.INSTANCE.closeTab());
+         Image img = new Image(new ImageResource2x(ThemeResources.INSTANCE.closeTab2x()));
          img.setStylePrimaryName(styles_.closeTabButton());
          img.addStyleName(ThemeStyles.INSTANCE.handCursor());
          contentPanel_.add(img);
