@@ -74,6 +74,7 @@ import org.rstudio.core.client.dom.DomUtils.ElementPredicate;
 import org.rstudio.core.client.events.EditorKeybindingsChangedEvent;
 import org.rstudio.core.client.events.RStudioKeybindingsChangedEvent;
 import org.rstudio.core.client.js.JsUtil;
+import org.rstudio.core.client.resources.ImageResource2x;
 import org.rstudio.core.client.theme.RStudioDataGridResources;
 import org.rstudio.core.client.theme.RStudioDataGridStyle;
 import org.rstudio.core.client.theme.res.ThemeResources;
@@ -81,7 +82,6 @@ import org.rstudio.studio.client.RStudioGinjector;
 import org.rstudio.studio.client.application.events.EventBus;
 import org.rstudio.studio.client.common.GlobalDisplay;
 import org.rstudio.studio.client.common.HelpLink;
-import org.rstudio.studio.client.workbench.AddinsMRUList;
 import org.rstudio.studio.client.workbench.addins.Addins.RAddin;
 import org.rstudio.studio.client.workbench.addins.Addins.RAddins;
 import org.rstudio.studio.client.workbench.addins.AddinsCommandManager;
@@ -480,7 +480,7 @@ public class ModifyKeyboardShortcutsWidget extends ModalDialogBase
          @Override
          public void execute(EditorKeyBindings bindings)
          {
-            events_.fireEvent(new AddinsKeyBindingsChangedEvent(bindings));
+            events_.fireEventToAllSatellites(new AddinsKeyBindingsChangedEvent(bindings));
          }
       });
       
@@ -494,8 +494,7 @@ public class ModifyKeyboardShortcutsWidget extends ModalDialogBase
                           AddinsServerOperations addinsServer,
                           Commands commands,
                           GlobalDisplay globalDisplay,
-                          EventBus events,
-                          AddinsMRUList mruAddins)
+                          EventBus events)
    {
       editorCommands_ = editorCommands;
       appCommands_ = appCommands;
@@ -504,7 +503,6 @@ public class ModifyKeyboardShortcutsWidget extends ModalDialogBase
       commands_ = commands;
       globalDisplay_ = globalDisplay;
       events_ = events;
-      mruAddins_ = mruAddins;
    }
    
    private void addColumns()
@@ -1287,7 +1285,7 @@ public class ModifyKeyboardShortcutsWidget extends ModalDialogBase
       
       embedIcon(
             shortcutCell,
-            ThemeResources.INSTANCE.syntaxInfo(),
+            new ImageResource2x(ThemeResources.INSTANCE.syntaxInfo2x()),
             "Masked by RStudio command: ",
             maskedIndex);
       
@@ -1301,7 +1299,7 @@ public class ModifyKeyboardShortcutsWidget extends ModalDialogBase
       
       embedIcon(
             shortcutCell,
-            ThemeResources.INSTANCE.syntaxWarning(),
+            new ImageResource2x(ThemeResources.INSTANCE.syntaxWarning2x()),
             "Conflicts with command: ",
             maskedIndex);
       
@@ -1452,7 +1450,6 @@ public class ModifyKeyboardShortcutsWidget extends ModalDialogBase
    private Commands commands_;
    private GlobalDisplay globalDisplay_;
    private EventBus events_;
-   private AddinsMRUList mruAddins_;
    
    // Resources, etc ----
    public interface Resources extends RStudioDataGridResources
