@@ -45,6 +45,7 @@ import org.rstudio.studio.client.projects.events.OpenProjectFileEvent;
 import org.rstudio.studio.client.projects.events.OpenProjectFileHandler;
 import org.rstudio.studio.client.projects.events.OpenProjectNewWindowEvent;
 import org.rstudio.studio.client.projects.events.OpenProjectNewWindowHandler;
+import org.rstudio.studio.client.projects.events.RequestOpenProjectEvent;
 import org.rstudio.studio.client.projects.events.SwitchToProjectEvent;
 import org.rstudio.studio.client.projects.events.SwitchToProjectHandler;
 import org.rstudio.studio.client.projects.events.NewProjectEvent;
@@ -84,7 +85,8 @@ public class Projects implements OpenProjectFileHandler,
                                  OpenProjectErrorHandler,
                                  OpenProjectNewWindowHandler,
                                  NewProjectEvent.Handler,
-                                 OpenProjectEvent.Handler
+                                 OpenProjectEvent.Handler,
+                                 RequestOpenProjectEvent.Handler
 {
    public interface Binder extends CommandBinder<Commands, Projects> {}
    
@@ -130,6 +132,7 @@ public class Projects implements OpenProjectFileHandler,
       eventBus.addHandler(OpenProjectNewWindowEvent.TYPE, this);
       eventBus.addHandler(NewProjectEvent.TYPE, this);
       eventBus.addHandler(OpenProjectEvent.TYPE, this);
+      eventBus.addHandler(RequestOpenProjectEvent.TYPE, this);
       
       eventBus.addHandler(SessionInitEvent.TYPE, new SessionInitHandler() {
          public void onSessionInit(SessionInitEvent sie)
@@ -210,7 +213,7 @@ public class Projects implements OpenProjectFileHandler,
    {
       handleNewProject(false, true);
    }
-
+   
    private void handleNewProject(boolean forceSaveAll, 
                                  final boolean allowOpenInNewWindow)
    {
@@ -315,13 +318,6 @@ public class Projects implements OpenProjectFileHandler,
                                           newProject.getCreateGitRepo());
                }
                
-               if (newProject.getUsePackrat() !=
-                   uiPrefs.newProjUsePackrat().getValue())
-               {
-                  uiPrefs.newProjUsePackrat().setGlobalValue(
-                                          newProject.getUsePackrat());
-               }
-               
                // call the server -- in all cases continue on with
                // creating the project (swallow errors updating the pref)
                projServer_.setUiPrefs(
@@ -409,7 +405,6 @@ public class Projects implements OpenProjectFileHandler,
          @Override
          public void onExecute(final Command continuation)
          {
-            
             // Validate the package name if we're creating a package
             if (newProject.getNewPackageOptions() != null)
             {
@@ -435,6 +430,7 @@ public class Projects implements OpenProjectFileHandler,
                      newProject.getProjectFile(),
                      newProject.getNewPackageOptions(),
                      newProject.getNewShinyAppOptions(),
+                     newProject.getProjectTemplateOptions(),
                      new VoidServerRequestCallback(indicator)
                      {
                         @Override
@@ -605,6 +601,20 @@ public class Projects implements OpenProjectFileHandler,
                             event.getAllowOpenInNewWindow());
    }
    
+   @Override
+   public void onRequestOpenProjectEvent(RequestOpenProjectEvent event)
+   {
+      String projFile = event.getProjectFile();
+      if (event.isNewSession())
+      {
+         eventBus_.fireEvent(new OpenProjectNewWindowEvent(projFile, null));
+      }
+      else
+      {
+         eventBus_.fireEvent(new SwitchToProjectEvent(projFile));
+      }
+   }
+   
    @Handler
    public void onOpenProjectInNewWindow()
    {
