@@ -1,7 +1,7 @@
 /*
  * ClientEventDispatcher.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2009-17 by RStudio, Inc.
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -22,6 +22,7 @@ import com.google.gwt.core.client.Scheduler;
 import com.google.gwt.core.client.Scheduler.RepeatingCommand;
 
 import org.rstudio.core.client.files.FileSystemItem;
+import org.rstudio.core.client.files.filedialog.events.OpenFileDialogEvent;
 import org.rstudio.core.client.js.JsObject;
 import org.rstudio.core.client.jsonrpc.RpcObjectList;
 import org.rstudio.studio.client.application.events.*;
@@ -46,6 +47,7 @@ import org.rstudio.studio.client.common.debugging.model.ErrorHandlerType;
 import org.rstudio.studio.client.common.debugging.model.UnhandledError;
 import org.rstudio.studio.client.common.dependencies.events.InstallShinyEvent;
 import org.rstudio.studio.client.common.rpubs.events.RPubsUploadStatusEvent;
+import org.rstudio.studio.client.common.rstudioapi.events.RStudioAPIShowDialogEvent;
 import org.rstudio.studio.client.common.sourcemarkers.SourceMarker;
 import org.rstudio.studio.client.common.synctex.events.SynctexEditFileEvent;
 import org.rstudio.studio.client.common.synctex.model.SourceLocation;
@@ -54,12 +56,18 @@ import org.rstudio.studio.client.events.EditorCommandEvent;
 import org.rstudio.studio.client.htmlpreview.events.HTMLPreviewCompletedEvent;
 import org.rstudio.studio.client.htmlpreview.events.HTMLPreviewOutputEvent;
 import org.rstudio.studio.client.htmlpreview.events.HTMLPreviewStartedEvent;
+import org.rstudio.studio.client.htmlpreview.events.ShowPageViewerEvent;
+import org.rstudio.studio.client.htmlpreview.model.HTMLPreviewParams;
 import org.rstudio.studio.client.htmlpreview.model.HTMLPreviewResult;
+import org.rstudio.studio.client.packages.events.PackageExtensionIndexingCompletedEvent;
 import org.rstudio.studio.client.projects.events.FollowUserEvent;
 import org.rstudio.studio.client.projects.events.OpenProjectErrorEvent;
 import org.rstudio.studio.client.projects.events.ProjectAccessRevokedEvent;
+import org.rstudio.studio.client.projects.events.ProjectTemplateRegistryUpdatedEvent;
 import org.rstudio.studio.client.projects.events.ProjectUserChangedEvent;
+import org.rstudio.studio.client.projects.events.RequestOpenProjectEvent;
 import org.rstudio.studio.client.projects.model.OpenProjectError;
+import org.rstudio.studio.client.projects.model.ProjectTemplateRegistry;
 import org.rstudio.studio.client.projects.model.ProjectUser;
 import org.rstudio.studio.client.rmarkdown.events.ChunkExecStateChangedEvent;
 import org.rstudio.studio.client.rmarkdown.events.ChunkPlotRefreshFinishedEvent;
@@ -74,11 +82,8 @@ import org.rstudio.studio.client.rmarkdown.events.RmdRenderCompletedEvent;
 import org.rstudio.studio.client.rmarkdown.events.RmdRenderOutputEvent;
 import org.rstudio.studio.client.rmarkdown.events.RmdRenderStartedEvent;
 import org.rstudio.studio.client.rmarkdown.events.RmdShinyDocStartedEvent;
-import org.rstudio.studio.client.rmarkdown.events.RmdTemplateDiscoveredEvent;
-import org.rstudio.studio.client.rmarkdown.events.RmdTemplateDiscoveryCompletedEvent;
 import org.rstudio.studio.client.rmarkdown.events.WebsiteFileSavedEvent;
 import org.rstudio.studio.client.rmarkdown.model.RmdChunkOutput;
-import org.rstudio.studio.client.rmarkdown.model.RmdDiscoveredTemplate;
 import org.rstudio.studio.client.rmarkdown.model.RmdRenderResult;
 import org.rstudio.studio.client.rmarkdown.model.RmdShinyDocInfo;
 import org.rstudio.studio.client.rsconnect.events.EnableRStudioConnectUIEvent;
@@ -86,7 +91,9 @@ import org.rstudio.studio.client.rsconnect.events.RSConnectDeploymentCompletedEv
 import org.rstudio.studio.client.rsconnect.events.RSConnectDeploymentFailedEvent;
 import org.rstudio.studio.client.rsconnect.events.RSConnectDeploymentOutputEvent;
 import org.rstudio.studio.client.server.Bool;
+import org.rstudio.studio.client.server.model.RequestDocumentSaveEvent;
 import org.rstudio.studio.client.shiny.events.ShinyApplicationStatusEvent;
+import org.rstudio.studio.client.shiny.events.ShinyFrameNavigatedEvent;
 import org.rstudio.studio.client.shiny.model.ShinyApplicationParams;
 import org.rstudio.studio.client.workbench.addins.Addins.RAddins;
 import org.rstudio.studio.client.workbench.addins.events.AddinRegistryUpdatedEvent;
@@ -105,6 +112,7 @@ import org.rstudio.studio.client.workbench.views.connections.events.ConnectionLi
 import org.rstudio.studio.client.workbench.views.connections.events.ConnectionOpenedEvent;
 import org.rstudio.studio.client.workbench.views.connections.events.ConnectionUpdatedEvent;
 import org.rstudio.studio.client.workbench.views.connections.events.EnableConnectionsEvent;
+import org.rstudio.studio.client.workbench.views.connections.events.NewConnectionDialogUpdatedEvent;
 import org.rstudio.studio.client.workbench.views.connections.model.Connection;
 import org.rstudio.studio.client.workbench.views.connections.model.ConnectionId;
 import org.rstudio.studio.client.workbench.views.console.events.*;
@@ -135,6 +143,7 @@ import org.rstudio.studio.client.workbench.views.output.sourcecpp.model.SourceCp
 import org.rstudio.studio.client.workbench.views.packages.events.PackageStateChangedEvent;
 import org.rstudio.studio.client.workbench.views.packages.events.LoadedPackageUpdatesEvent;
 import org.rstudio.studio.client.workbench.views.packages.events.PackageStatusChangedEvent;
+import org.rstudio.studio.client.workbench.views.packages.model.PackageProvidedExtensions;
 import org.rstudio.studio.client.workbench.views.packages.model.PackageState;
 import org.rstudio.studio.client.workbench.views.packages.model.PackageStatus;
 import org.rstudio.studio.client.workbench.views.plots.events.LocatorEvent;
@@ -144,6 +153,7 @@ import org.rstudio.studio.client.workbench.views.plots.model.PlotsState;
 import org.rstudio.studio.client.workbench.views.presentation.events.PresentationPaneRequestCompletedEvent;
 import org.rstudio.studio.client.workbench.views.presentation.events.ShowPresentationPaneEvent;
 import org.rstudio.studio.client.workbench.views.presentation.model.PresentationState;
+import org.rstudio.studio.client.workbench.views.source.editors.explorer.events.ObjectExplorerEvent;
 import org.rstudio.studio.client.workbench.views.source.editors.profiler.RprofEvent;
 import org.rstudio.studio.client.workbench.views.source.events.CodeBrowserNavigationEvent;
 import org.rstudio.studio.client.workbench.views.source.events.CollabEditEndedEvent;
@@ -157,6 +167,13 @@ import org.rstudio.studio.client.workbench.views.source.events.ShowDataEvent;
 import org.rstudio.studio.client.workbench.views.source.events.SourceExtendedTypeDetectedEvent;
 import org.rstudio.studio.client.workbench.views.source.model.ContentItem;
 import org.rstudio.studio.client.workbench.views.source.model.DataItem;
+import org.rstudio.studio.client.workbench.views.terminal.events.ActivateNamedTerminalEvent;
+import org.rstudio.studio.client.workbench.views.terminal.events.AddTerminalEvent;
+import org.rstudio.studio.client.workbench.views.terminal.events.ClearTerminalEvent;
+import org.rstudio.studio.client.workbench.views.terminal.events.RemoveTerminalEvent;
+import org.rstudio.studio.client.workbench.views.terminal.events.SendToTerminalEvent;
+import org.rstudio.studio.client.workbench.views.terminal.events.TerminalCwdEvent;
+import org.rstudio.studio.client.workbench.views.terminal.events.TerminalSubprocEvent;
 import org.rstudio.studio.client.workbench.views.vcs.common.events.AskPassEvent;
 import org.rstudio.studio.client.workbench.views.vcs.common.events.VcsRefreshEvent;
 import org.rstudio.studio.client.workbench.views.vcs.common.events.VcsRefreshEvent.Reason;
@@ -358,8 +375,7 @@ public class ClientEventDispatcher
          {
             ServerConsoleOutputEvent.Data data = event.getData();
             eventBus_.fireEvent(new ServerConsoleOutputEvent(data.getHandle(),
-                                                            data.getOutput(),
-                                                            data.isError()));
+                                                            data.getOutput()));
          }
          else if (type.equals(ClientEvent.ConsoleProcessPrompt))
          {
@@ -605,15 +621,6 @@ public class ClientEventDispatcher
             RmdRenderResult result = event.getData();
             eventBus_.fireEvent(new RmdRenderCompletedEvent(result));
          }
-         else if (type.equals(ClientEvent.RmdTemplateDiscovered))
-         {
-            RmdDiscoveredTemplate template = event.getData();
-            eventBus_.fireEvent(new RmdTemplateDiscoveredEvent(template));
-         }
-         else if (type.equals(ClientEvent.RmdTemplateDiscoveryCompleted))
-         {
-            eventBus_.fireEvent(new RmdTemplateDiscoveryCompletedEvent());
-         }
          else if (type.equals(ClientEvent.RmdShinyDocStarted))
          {
             RmdShinyDocInfo docInfo = event.getData();
@@ -846,6 +853,96 @@ public class ClientEventDispatcher
             ChunkExecStateChangedEvent.Data data = event.getData();
             eventBus_.fireEvent(new ChunkExecStateChangedEvent(data));
          }
+         else if (type.equals(ClientEvent.NavigateShinyFrame))
+         {
+            ShinyFrameNavigatedEvent.Data data = event.getData();
+            eventBus_.fireEvent(new ShinyFrameNavigatedEvent(data));
+         }
+         else if (type.equals(ClientEvent.UpdateNewConnectionDialog))
+         {
+            NewConnectionDialogUpdatedEvent.Data data = event.getData();
+            eventBus_.fireEvent(new NewConnectionDialogUpdatedEvent(data));
+         }
+         else if (type.equals(ClientEvent.ProjectTemplateRegistryUpdated))
+         {
+            ProjectTemplateRegistry data = event.getData();
+            eventBus_.fireEvent(new ProjectTemplateRegistryUpdatedEvent(data));
+         }
+         else if (type.equals(ClientEvent.TerminalSubProcs))
+         {
+            TerminalSubprocEvent.Data data = event.getData();
+            eventBus_.fireEvent(new TerminalSubprocEvent(data));
+         }
+         else if (type.equals(ClientEvent.PackageExtensionIndexingCompleted))
+         {
+            PackageProvidedExtensions.Data data = event.getData();
+            eventBus_.fireEvent(new PackageExtensionIndexingCompletedEvent(data));
+         }
+         else if (type.equals(ClientEvent.RStudioAPIShowDialog))
+         {
+            RStudioAPIShowDialogEvent.Data data = event.getData();
+            eventBus_.fireEvent(new RStudioAPIShowDialogEvent(data));
+         }
+         else if (type.equals(ClientEvent.ObjectExplorerEvent))
+         {
+            ObjectExplorerEvent.Data data = event.getData();
+            eventBus_.fireEvent(new ObjectExplorerEvent(data));
+         }
+         else if (type.equals(ClientEvent.SendToTerminal))
+         {
+            SendToTerminalEvent.Data data = event.getData();
+            eventBus_.fireEvent(new SendToTerminalEvent(data));
+         }
+         else if (type.equals(ClientEvent.ClearTerminal))
+         {
+            ClearTerminalEvent.Data data = event.getData();
+            eventBus_.fireEvent(new ClearTerminalEvent(data));
+         }
+         else if (type.equals(ClientEvent.AddTerminal))
+         {
+            AddTerminalEvent.Data data = event.getData();
+            eventBus_.fireEvent(new AddTerminalEvent(data));
+         }
+         else if (type.equals(ClientEvent.RemoveTerminal))
+         {
+            RemoveTerminalEvent.Data data = event.getData();
+            eventBus_.fireEvent(new RemoveTerminalEvent(data));
+         }
+         else if (type.equals(ClientEvent.ActivateTerminal))
+         {
+            ActivateNamedTerminalEvent.Data data = event.getData();
+            eventBus_.fireEvent(new ActivateNamedTerminalEvent(data));
+         }
+         else if (type.equals(ClientEvent.TerminalCwd))
+         {
+            TerminalCwdEvent.Data data = event.getData();
+            eventBus_.fireEvent(new TerminalCwdEvent(data));
+         }
+         else if (type.equals(ClientEvent.AdminNotification))
+         {
+            AdminNotification notification = event.getData();
+            eventBus_.fireEvent(new AdminNotificationEvent(notification));
+         }
+         else if (type.equals(ClientEvent.RequestDocumentSave))
+         {
+            RequestDocumentSaveEvent.Data data = event.getData();
+            eventBus_.fireEvent(new RequestDocumentSaveEvent(data));
+         }
+         else if (type.equals(ClientEvent.RequestOpenProject))
+         {
+            RequestOpenProjectEvent.Data data = event.getData();
+            eventBus_.fireEvent(new RequestOpenProjectEvent(data));
+         }
+         else if (type.equals(ClientEvent.OpenFileDialog))
+         {
+            OpenFileDialogEvent.Data data = event.getData();
+            eventBus_.fireEvent(new OpenFileDialogEvent(data));
+         }
+         else if (type.equals(ClientEvent.ShowPageViewer))
+         {
+            HTMLPreviewParams params = event.getData();
+            eventBus_.fireEvent(new ShowPageViewerEvent(params));
+         }
          else
          {
             GWT.log("WARNING: Server event not dispatched: " + type, null);
