@@ -1,7 +1,7 @@
 /*
  * UIPrefsAccessor.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2009-17 by RStudio, Inc.
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,6 +14,11 @@
  */
 package org.rstudio.studio.client.workbench.prefs.model;
 
+import java.util.ArrayList;
+import java.util.List;
+
+import org.rstudio.core.client.JsArrayUtil;
+import org.rstudio.core.client.VirtualConsole;
 import org.rstudio.core.client.js.JsObject;
 import org.rstudio.studio.client.application.Desktop;
 import org.rstudio.studio.client.notebook.CompileNotebookPrefs;
@@ -305,11 +310,21 @@ public class UIPrefsAccessor extends Prefs
       return integer("truncate_long_lines_in_console", 1000);
    }
    
+   public PrefValue<Integer> consoleAnsiMode()
+   {
+      return integer("ansi_console_mode", VirtualConsole.ANSI_COLOR_ON);
+   }
+   
    public PrefValue<Boolean> showInlineToolbarForRCodeChunks()
    {
       return bool("show_inline_toolbar_for_r_code_chunks", true);
    }
    
+   public PrefValue<Boolean> highlightCodeChunks()
+   {
+      return bool("highlight_code_chunks", true);
+   }
+   
    public PrefValue<Boolean> saveAllBeforeBuild()
    {
       return bool("save_files_before_build", false);
@@ -319,6 +334,11 @@ public class UIPrefsAccessor extends Prefs
    {
       return dbl("font_size_points", 10.0);
    }
+   
+   public PrefValue<Boolean> useLigatures()
+   {
+      return bool("use_ligatures", false);
+   }
 
    public PrefValue<String> theme()
    {
@@ -394,11 +414,6 @@ public class UIPrefsAccessor extends Prefs
       return bool("new_proj_git_init", false);
    }
    
-   public PrefValue<Boolean> newProjUsePackrat()
-   {
-      return bool("new_proj_use_packrat", false);
-   }
-   
    public PrefValue<String> defaultSweaveEngine()
    {
       return string("default_sweave_engine", "Sweave");
@@ -549,12 +564,6 @@ public class UIPrefsAccessor extends Prefs
    {
       return bool("show_publish_diagnostics", false);
    }
-   
-   public PrefValue<String> connectionsDbInterface()
-   {
-      return string("connections_db_interface", 
-                    ConnectionOptions.DB_INTERFACE_DPLYR);
-   }
      
    public PrefValue<Boolean> showRmdChunkOutputInline()
    {
@@ -581,9 +590,66 @@ public class UIPrefsAccessor extends Prefs
       return bool("hide_console_on_chunk_execute", true);
    }
    
-   public PrefValue<Boolean> executeMultiLineStatements()
+   public static final String EXECUTE_LINE      = "line";
+   public static final String EXECUTE_STATEMENT = "statement";
+   public static final String EXECUTE_PARAGRAPH = "paragraph";
+   
+   public PrefValue<String> executionBehavior()
    {
-      return bool("execute_multi_line_statements", true);
+      return string("execution_behavior", EXECUTE_STATEMENT);
+   }
+   
+   public PrefValue<Boolean> showTerminalTab()
+   {
+      return bool("show_terminal_tab", true);
+   }
+   
+   public PrefValue<Boolean> terminalLocalEcho()
+   {
+      return bool("terminal_local_echo", true);
+   }
+   
+   public PrefValue<Boolean> terminalUseWebsockets()
+   {
+      return bool("terminal_websockets", true);
+   }
+   
+   public PrefValue<Boolean> terminalAutoClose()
+   {
+      return bool("terminal_autoclose", true);
+   }
+
+   public PrefValue<Boolean> terminalTrackEnvironment()
+   {
+      return bool("terminal_track_env", true);
+   }
+
+   public static final int BUSY_DETECT_ALWAYS = 0;
+   public static final int BUSY_DETECT_NEVER = 1;
+   public static final int BUSY_DETECT_WHITELIST = 2;
+
+   public PrefValue<Integer> terminalBusyMode()
+   {
+      return integer("busy_detection", BUSY_DETECT_ALWAYS);
+   }
+
+   public PrefValue<JsArrayString> terminalBusyWhitelist()
+   {
+      List<String> defArray = new ArrayList<String>();
+      defArray.add("tmux");
+      defArray.add("screen");
+      JsArrayString defJsArray = JsArrayUtil.toJsArrayString(defArray);
+
+      return object("busy_whitelist", defJsArray);
+   }
+ 
+   public static final String KNIT_DIR_DEFAULT = "default";
+   public static final String KNIT_DIR_CURRENT = "current";
+   public static final String KNIT_DIR_PROJECT = "project";
+   
+   public PrefValue<String> knitWorkingDir()
+   {
+      return string("knit_working_dir", KNIT_DIR_DEFAULT);
    }
    
    public static final String DOC_OUTLINE_SHOW_SECTIONS_ONLY = "show_sections_only";
@@ -604,6 +670,21 @@ public class UIPrefsAccessor extends Prefs
       return string("latex_preview_on_cursor_idle", LATEX_PREVIEW_SHOW_ALWAYS);
    }
    
+   public PrefValue<Boolean> wrapTabNavigation()
+   {
+      return bool("wrap_tab_navigation", false);
+   }
+
+   public PrefValue<String> getFlatTheme()
+   {
+      return string("flat_theme", "default");
+   }
+   
+   public PrefValue<Boolean> gitDiffIgnoreWhitespace()
+   {
+      return bool("git_diff_ignore_whitespace", false);
+   }
+   
    private String getDefaultPdfPreview()
    {
       if (Desktop.isDesktop())
