@@ -1,7 +1,7 @@
 /*
  * Shell.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2009-17 by RStudio, Inc.
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -136,11 +136,13 @@ public class Shell implements ConsoleHistoryAddedEvent.Handler,
       keyDownPreviewHandlers_ = new ArrayList<KeyDownPreviewHandler>() ;
       keyPressPreviewHandlers_ = new ArrayList<KeyPressPreviewHandler>() ;
 
-      InputKeyDownHandler handler = new InputKeyDownHandler() ;
+      InputKeyHandler handler = new InputKeyHandler() ;
+
       // This needs to be a capturing key down handler or else Ace will have
       // handled the event before we had a chance to prevent it
-      view_.addCapturingKeyDownHandler(handler) ;
-      view_.addKeyPressHandler(handler) ;
+      view_.addCapturingKeyDownHandler(handler);
+      view_.addKeyPressHandler(handler);
+      view_.addCapturingKeyUpHandler(handler);
       
       eventBus.addHandler(ConsoleHistoryAddedEvent.TYPE, this);
       eventBus.addHandler(ConsoleInputEvent.TYPE, this); 
@@ -170,8 +172,9 @@ public class Shell implements ConsoleHistoryAddedEvent.Handler,
       addKeyDownPreviewHandler(completionManager) ;
       addKeyPressPreviewHandler(completionManager) ;
       
-      addKeyDownPreviewHandler(new HistoryCompletionManager(
-            view_.getInputEditorDisplay(), server));
+      historyCompletion_ = new HistoryCompletionManager(
+            view_.getInputEditorDisplay(), server);
+      addKeyDownPreviewHandler(historyCompletion_);
       
       // we need to explicitly connect a paste handler on Desktop
       // to ensure the completion popup is dismissed in shell on paste
@@ -482,14 +485,26 @@ public class Shell implements ConsoleHistoryAddedEvent.Handler,
          historyManager_.addToHistory(event.getCode());
    }
 
-   private final class InputKeyDownHandler implements KeyDownHandler,
-                                                      KeyPressHandler
+   private final class InputKeyHandler implements KeyDownHandler,
+                                                  KeyPressHandler,
+                                                  KeyUpHandler
    {
       public void onKeyDown(KeyDownEvent event)
       {
          int keyCode = event.getNativeKeyCode();
+         
+         // typically we allow all the handlers to process the key; however,
+         // this behavior is suppressed when we're incrementally searching the
+         // history so we don't stack two kinds of completion popups
+         ArrayList<KeyDownPreviewHandler> handlers = keyDownPreviewHandlers_;
+         if (historyCompletion_.getMode() == 
+               HistoryCompletionManager.PopupMode.PopupIncremental)
+         {
+            handlers = new ArrayList<KeyDownPreviewHandler>();
+            handlers.add(historyCompletion_);
+         }
 
-         for (KeyDownPreviewHandler handler : keyDownPreviewHandlers_)
+         for (KeyDownPreviewHandler handler : handlers)
          {
             if (handler.previewKeyDown(event.getNativeEvent()))
             {
@@ -504,7 +519,8 @@ public class Shell implements ConsoleHistoryAddedEvent.Handler,
 
          int modifiers = KeyboardShortcut.getModifierValue(event.getNativeEvent());
 
-         if (event.isUpArrow() && modifiers == 0)
+         if ((event.isUpArrow() && modifiers == 0) ||
+             (keyCode == 'P'    && modifiers == KeyboardShortcut.CTRL))
          {
             if ((input_.getCurrentLineNum() == 0) || input_.isCursorAtEnd())
             {
@@ -514,7 +530,8 @@ public class Shell implements ConsoleHistoryAddedEvent.Handler,
                navigateHistory(-1);
             }
          }
-         else if (event.isDownArrow() && modifiers == 0)
+         else if ((event.isDownArrow() && modifiers == 0) ||
+                  (keyCode == 'N'      && modifiers == KeyboardShortcut.CTRL))
          {
             if ((input_.getCurrentLineNum() == input_.getCurrentLineCount() - 1)
                 || input_.isCursorAtEnd())
@@ -525,7 +542,10 @@ public class Shell implements ConsoleHistoryAddedEvent.Handler,
                navigateHistory(1);
             }
          }
-         else if (keyCode == KeyCodes.KEY_ENTER && modifiers == 0)
+         else if (keyCode == KeyCodes.KEY_ENTER && (
+                     modifiers == 0 ||
+                     modifiers == KeyboardShortcut.CTRL ||
+                     modifiers == KeyboardShortcut.META))
          {
             event.preventDefault();
             event.stopPropagation();
@@ -600,7 +620,18 @@ public class Shell implements ConsoleHistoryAddedEvent.Handler,
 
       public void onKeyPress(KeyPressEvent event)
       {
-         for (KeyPressPreviewHandler handler : keyPressPreviewHandlers_)
+         // typically we allow all the handlers to process the key; however,
+         // this behavior is suppressed when we're incrementally searching the
+         // history so we don't stack two kinds of completion popups
+         ArrayList<KeyPressPreviewHandler> handlers = keyPressPreviewHandlers_;
+         if (historyCompletion_.getMode() == 
+               HistoryCompletionManager.PopupMode.PopupIncremental)
+         {
+            handlers = new ArrayList<KeyPressPreviewHandler>();
+            handlers.add(historyCompletion_);
+         }
+
+         for (KeyPressPreviewHandler handler : handlers)
          {
             if (handler.previewKeyPress(event.getCharCode()))
             {
@@ -611,6 +642,20 @@ public class Shell implements ConsoleHistoryAddedEvent.Handler,
          }
       }
 
+      @Override
+      public void onKeyUp(KeyUpEvent event)
+      {
+         if (event.isAnyModifierKeyDown())
+            return;
+         if (KeyCodeEvent.isArrow(event.getNativeKeyCode()))
+            return;
+         if (historyCompletion_.getMode() == 
+               HistoryCompletionManager.PopupMode.PopupIncremental)
+         {
+            historyCompletion_.beginSearch();
+         }
+      }
+
       @SuppressWarnings("unused")
       private boolean lastKeyCodeWasZero_;
    }
@@ -682,6 +727,8 @@ public class Shell implements ConsoleHistoryAddedEvent.Handler,
    private final InputEditorDisplay input_ ;
    private final ArrayList<KeyDownPreviewHandler> keyDownPreviewHandlers_ ;
    private final ArrayList<KeyPressPreviewHandler> keyPressPreviewHandlers_ ;
+   private final HistoryCompletionManager historyCompletion_;
+   
    // indicates whether the next command should be added to history
    private boolean addToHistory_ ;
    private String lastPromptText_ ;
