@@ -19,6 +19,7 @@ import org.rstudio.core.client.Debug;
 import org.rstudio.core.client.Size;
 import org.rstudio.core.client.dom.DomMetrics;
 import org.rstudio.core.client.dom.DomUtils;
+import org.rstudio.core.client.resources.ImageResource2x;
 import org.rstudio.core.client.widget.GridViewerFrame;
 import org.rstudio.core.client.widget.Operation;
 import org.rstudio.core.client.widget.OperationWithInput;
@@ -26,6 +27,7 @@ import org.rstudio.core.client.widget.ProgressIndicator;
 import org.rstudio.core.client.widget.ProgressIndicatorDelay;
 import org.rstudio.studio.client.RStudioGinjector;
 import org.rstudio.studio.client.common.GlobalDisplay;
+import org.rstudio.studio.client.common.HelpLink;
 import org.rstudio.studio.client.common.reditor.EditorLanguage;
 import org.rstudio.studio.client.server.ServerError;
 import org.rstudio.studio.client.server.ServerRequestCallback;
@@ -257,7 +259,9 @@ public class DataImport extends Composite
    
    PushButton makeCopyButton()
    {
-      return new PushButton(new Image(dataImportResources_.copyImage()), new ClickHandler()
+      return new PushButton(new Image(new ImageResource2x(
+         dataImportResources_.copyImage(),
+         dataImportResources_.copyImage2x())), new ClickHandler()
       {
          @Override
          public void onClick(ClickEvent arg0)
@@ -720,4 +724,8 @@ public class DataImport extends Composite
          return !column.col_name && column.col_type != 'rownames';
       });
    }-*/;
+   
+   public HelpLink getHelpLink() {
+      return dataImportOptionsUi_.getHelpLink();
+   }
 }
