@@ -15,6 +15,7 @@
 
 package org.rstudio.studio.client.workbench.views.help;
 
+import com.google.gwt.core.client.GWT;
 import com.google.gwt.core.client.JavaScriptObject;
 import com.google.gwt.core.client.Scheduler;
 import com.google.gwt.core.client.Scheduler.RepeatingCommand;
@@ -32,6 +33,8 @@ import com.google.gwt.event.logical.shared.ResizeEvent;
 import com.google.gwt.event.logical.shared.ResizeHandler;
 import com.google.gwt.event.shared.HandlerRegistration;
 import com.google.gwt.http.client.URL;
+import com.google.gwt.resources.client.ClientBundle;
+import com.google.gwt.resources.client.CssResource;
 import com.google.gwt.user.client.Window;
 import com.google.gwt.user.client.ui.*;
 import com.google.inject.Inject;
@@ -48,17 +51,21 @@ import org.rstudio.core.client.dom.IFrameElementEx;
 import org.rstudio.core.client.dom.WindowEx;
 import org.rstudio.core.client.events.NativeKeyDownEvent;
 import org.rstudio.core.client.files.FileSystemItem;
+import org.rstudio.core.client.theme.res.ThemeStyles;
 import org.rstudio.core.client.widget.CanFocus;
 import org.rstudio.core.client.widget.FindTextBox;
 import org.rstudio.core.client.widget.MessageDialog;
-import org.rstudio.core.client.widget.RStudioFrame;
+import org.rstudio.core.client.widget.RStudioThemedFrame;
 import org.rstudio.core.client.widget.SecondaryToolbar;
 import org.rstudio.core.client.widget.SmallButton;
 import org.rstudio.core.client.widget.Toolbar;
+import org.rstudio.core.client.widget.ToolbarButton;
+import org.rstudio.studio.client.application.events.EventBus;
 import org.rstudio.studio.client.common.AutoGlassPanel;
 import org.rstudio.studio.client.common.GlobalDisplay;
 import org.rstudio.studio.client.workbench.commands.Commands;
 import org.rstudio.studio.client.workbench.ui.WorkbenchPane;
+import org.rstudio.studio.client.workbench.views.console.events.SendToConsoleEvent;
 import org.rstudio.studio.client.workbench.views.help.Help.LinkMenu;
 import org.rstudio.studio.client.workbench.views.help.events.HelpNavigateEvent;
 import org.rstudio.studio.client.workbench.views.help.events.HelpNavigateHandler;
@@ -71,13 +78,15 @@ public class HelpPane extends WorkbenchPane
    @Inject
    public HelpPane(Provider<HelpSearch> searchProvider,
                    GlobalDisplay globalDisplay,
-                   Commands commands)
+                   Commands commands,
+                   EventBus events)
    {
       super("Help") ;
       
       searchProvider_ = searchProvider ;
       globalDisplay_ = globalDisplay;
       commands_ = commands;
+      events_ = events;
     
       MenuItem clear = commands.clearHelpHistory().createMenuItem(false);
       history_ = new ToolbarLinkMenu(12, true, null, new MenuItem[] { clear }) ;
@@ -96,9 +105,22 @@ public class HelpPane extends WorkbenchPane
    @Override 
    protected Widget createMainWidget()
    {
-      frame_ = new RStudioFrame();
+      frame_ = new RStudioThemedFrame(
+         null,
+         ".rstudio-themes-flat.editor_dark div#TOC,\n" +
+         ".rstudio-themes-flat.editor_dark h1,\n" +
+         ".rstudio-themes-flat.editor_dark h2,\n" +
+         ".rstudio-themes-flat.editor_dark h3,\n" +
+         ".rstudio-themes-flat.editor_dark h4,\n" +
+         ".rstudio-themes-flat.editor_dark table {\n" +
+         "  background: none !important;\n" +
+         "  color: white !important;\n" +
+         "}\n",
+         null,
+         false);
       frame_.setSize("100%", "100%");
       frame_.setStylePrimaryName("rstudio-HelpFrame");
+      frame_.addStyleName("ace_editor_theme");
       ElementIds.assignElementId(frame_.getElement(), ElementIds.HELP_FRAME);
 
       return new AutoGlassPanel(frame_);
@@ -192,18 +214,36 @@ public class HelpPane extends WorkbenchPane
    { 
       // determine whether this key-combination means we should focus find
       int mod = KeyboardShortcut.getModifierValue(e);
-      if ((mod == (BrowseCap.hasMetaKey() ? KeyboardShortcut.META
-                                          : KeyboardShortcut.CTRL)) &&
-           e.getKeyCode() == 'F')
+      if (mod == (BrowseCap.hasMetaKey() ? KeyboardShortcut.META
+                                         : KeyboardShortcut.CTRL))
       {
-         e.preventDefault();
-         e.stopPropagation();
-         WindowEx.get().focus();
-         findTextBox_.focus();
-         findTextBox_.selectAll();
-         return;
+         if (e.getKeyCode() == 'F')
+         {
+            e.preventDefault();
+            e.stopPropagation();
+            WindowEx.get().focus();
+            findTextBox_.focus();
+            findTextBox_.selectAll();
+            return;
+         }
+         else if (e.getKeyCode() == KeyCodes.KEY_ENTER)
+         {
+            // extract the selected code, if any
+            String code = frame_.getWindow().getSelectedText();
+            if (code.isEmpty())
+               return;
+            
+            // send it to the console
+            events_.fireEvent(new SendToConsoleEvent(
+                  code, 
+                  true, // execute
+                  false // focus
+                  ));
+            return;
+         }
       }
       
+      
       // don't let backspace perform browser back
       DomUtils.preventBackspaceCausingBrowserBack(e);
       
@@ -304,22 +344,31 @@ public class HelpPane extends WorkbenchPane
       toolbar.addRightWidget(searchProvider_.get().getSearchWidget());
 
       toolbar.addRightSeparator();
-      toolbar.addRightWidget(commands_.refreshHelp().createToolbarButton());
+
+      ToolbarButton refreshButton = commands_.refreshHelp().createToolbarButton();
+      refreshButton.addStyleName(ThemeStyles.INSTANCE.refreshToolbarButton());
+      toolbar.addRightWidget(refreshButton);
     
-      
       return toolbar;
    }
    
    @Override
    protected SecondaryToolbar createSecondaryToolbar()
    {
-      SecondaryToolbar toolbar = new SecondaryToolbar() ;
-      toolbar.addLeftPopupMenu(title_ = new Label(), history_.getMenu());
+      SecondaryToolbar toolbar = new SecondaryToolbar();
+
+      title_ = new Label();
+      title_.addStyleName(RES.styles().topicTitle());
+      toolbar.addLeftPopupMenu(title_, history_.getMenu());
+
+      ThemeStyles styles = ThemeStyles.INSTANCE;
+      toolbar.getWrapper().addStyleName(styles.tallerToolbarWrapper());
       
       if (isFindSupported())
       {
          final SmallButton btnNext = new SmallButton("&gt;", true);
          btnNext.setTitle("Find next (Enter)");
+         btnNext.addStyleName(RES.styles().topicNavigationButton());
          btnNext.setVisible(false);
          btnNext.addClickHandler(new ClickHandler() {
             @Override
@@ -331,6 +380,7 @@ public class HelpPane extends WorkbenchPane
          
          final SmallButton btnPrev = new SmallButton("&lt;", true);
          btnPrev.setTitle("Find previous");
+         btnPrev.addStyleName(RES.styles().topicNavigationButton());
          btnPrev.setVisible(false);
          btnPrev.addClickHandler(new ClickHandler() {
             @Override
@@ -342,6 +392,7 @@ public class HelpPane extends WorkbenchPane
          
          
          findTextBox_ = new FindTextBox("Find in Topic");
+         findTextBox_.addStyleName(RES.styles().findTopicTextbox());
          findTextBox_.setOverrideWidth(90);
          toolbar.addLeftWidget(findTextBox_);
          findTextBox_.addKeyUpHandler(new KeyUpHandler() { 
@@ -677,16 +728,32 @@ public class HelpPane extends WorkbenchPane
    	 }, 0);
    }-*/;
 
+   public interface Styles extends CssResource
+   {
+      String findTopicTextbox();
+      String topicNavigationButton();
+      String topicTitle();
+   }
+   
+   public interface Resources extends ClientBundle
+   {
+      @Source("HelpPane.css")
+      Styles styles();
+   }
+
+   private static final Resources RES = GWT.create(Resources.class);
+   static { RES.styles().ensureInjected(); }
 
    private final VirtualHistory navStack_ = new VirtualHistory() ;
    private final ToolbarLinkMenu history_ ;
  
    private Label title_ ;
-   private RStudioFrame frame_ ;
+   private RStudioThemedFrame frame_ ;
    private FindTextBox findTextBox_;
    private final Provider<HelpSearch> searchProvider_ ;
    private GlobalDisplay globalDisplay_;
    private final Commands commands_;
+   private final EventBus events_;
    private boolean navigated_;
    private boolean initialized_;
    private String targetUrl_;
