@@ -54,7 +54,6 @@ import org.rstudio.studio.client.workbench.views.BasePresenter;
 import org.rstudio.studio.client.workbench.views.console.events.ConsoleResetHistoryEvent;
 import org.rstudio.studio.client.workbench.views.console.events.ConsoleResetHistoryHandler;
 import org.rstudio.studio.client.workbench.views.console.events.SendToConsoleEvent;
-import org.rstudio.studio.client.workbench.views.history.History.Display.Mode;
 import org.rstudio.studio.client.workbench.views.history.events.FetchCommandsEvent;
 import org.rstudio.studio.client.workbench.views.history.events.FetchCommandsHandler;
 import org.rstudio.studio.client.workbench.views.history.events.HistoryEntriesAddedEvent;
@@ -415,12 +414,7 @@ public class History extends BasePresenter implements SelectionCommitHandler<Voi
    public void onSelected()
    {
       super.onSelected();
-      if (view_.getMode() == Mode.Recent)
-      {
-         view_.scrollToBottom();
-         view_.focusSearch();
-      }
-      
+      view_.focusSearch();
    }
 
    private String getSelectedCommands()
