@@ -87,17 +87,17 @@ public class ChunkOutputWidget extends Composite
    
    public interface Resources extends ClientBundle
    {
-      @Source("ExpandChunkIcon.png")
-      ImageResource expandChunkIcon();
+      @Source("ExpandChunkIcon_2x.png")
+      ImageResource expandChunkIcon2x();
       
-      @Source("CollapseChunkIcon.png")
-      ImageResource collapseChunkIcon();
+      @Source("CollapseChunkIcon_2x.png")
+      ImageResource collapseChunkIcon2x();
 
-      @Source("RemoveChunkIcon.png")
-      ImageResource removeChunkIcon();
+      @Source("RemoveChunkIcon_2x.png")
+      ImageResource removeChunkIcon2x();
 
-      @Source("PopoutChunkIcon.png")
-      ImageResource popoutIcon();
+      @Source("PopoutChunkIcon_2x.png")
+      ImageResource popoutIcon2x();
    }
    
    public interface ChunkStyle extends CssResource
@@ -127,6 +127,11 @@ public class ChunkOutputWidget extends Composite
          setCollapsedStyles();
 
       ChunkDataWidget.injectPagedTableResources();
+
+      clear_.addStyleName("rstudio-themes-inverts");
+      clear_.addStyleName("rstudio-classic-inverts");
+      expand_.addStyleName("rstudio-themes-inverts");
+      expand_.addStyleName("rstudio-classic-inverts");
       
       if (chunkOutputSize_ == ChunkOutputSize.Default)
       {
